package com.kalixia.intellij.plugins.je;

import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.startup.StartupManager;
import com.intellij.openapi.util.IconLoader;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;
import com.kalixia.intellij.plugins.je.browser.JEBrowserPanel;
import com.kalixia.intellij.plugins.je.browser.Settings;
import com.sleepycat.je.DatabaseException;
import org.jetbrains.annotations.NotNull;
import org.jdesktop.swingx.JXErrorPane;
import org.jdesktop.swingx.error.ErrorInfo;
import org.jdesktop.swingx.error.ErrorLevel;
import javax.swing.*;

public class JEWindow implements ProjectComponent {
    private Project project;
    private Settings settings;
    private ToolWindowManager twm;

    public JEWindow(Project project, Settings settings, ToolWindowManager twm) {
        this.project = project;
        this.settings = settings;
        this.twm = twm;
    }

    public void initComponent() {
    }

    public void disposeComponent() {
    }

    @NotNull
    public String getComponentName() {
        return "JEWindow";
    }

    public void projectOpened() {
        final JEBrowserPanel browser = new JEBrowserPanel(project, settings);
        ToolWindow myToolWindow = twm.registerToolWindow("JE", browser.getRootComponent(), ToolWindowAnchor.LEFT);
        myToolWindow.setTitle("Browser");
        myToolWindow.setIcon(IconLoader.getIcon("/oracle.ico"));

        StartupManager.getInstance(project).registerPostStartupActivity(new Runnable() {
            public void run() {
                for (Settings.StoreConfiguration store : settings.getStores())
                    try {
                        browser.addStore(store.getEnvironmentDirectory(), store.getStoreName(), false);
                    } catch (Exception e) {
                        JXErrorPane.showDialog(browser.getRootComponent(), new ErrorInfo(
                                "Add Berkeley DB (Java Edition) store",
                                e.getMessage(),
                                null, "ERROR", e, ErrorLevel.SEVERE, null)
                        );
                    }
            }
        });
    }

    public void projectClosed() {
        twm.unregisterToolWindow("Browser");
    }
}
