package com.kalixia.intellij.plugins.je.browser;

import com.intellij.openapi.fileChooser.FileTextField;
import com.intellij.openapi.fileChooser.FileChooserDescriptor;
import com.intellij.openapi.fileChooser.FileChooserDescriptorFactory;
import com.intellij.openapi.fileChooser.FileChooserDialog;
import com.intellij.openapi.fileChooser.FileChooserFactory;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.wm.ex.WindowManagerEx;
import com.intellij.openapi.vfs.VirtualFile;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.Environment;
import com.sleepycat.je.EnvironmentConfig;
import com.sleepycat.persist.StoreConfig;
import com.sleepycat.persist.raw.RawStore;
import org.jdesktop.swingx.JXErrorPane;
import org.jdesktop.swingx.error.ErrorInfo;
import org.jdesktop.swingx.error.ErrorLevel;
import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

public class AddEnvironmentDialog extends JDialog {
    private JEBrowserPanel browser;
    private JPanel contentPane;
    private JButton buttonOK;
    private JButton buttonCancel;
    private JTextField tfStoreName;
    private JButton btnEnvironmentDirectory;
    private JTextField tfEnvironmentDirectory;

    public AddEnvironmentDialog(final JEBrowserPanel browser) {
        this.browser = browser;
        setContentPane(contentPane);
        setModal(true);
        getRootPane().setDefaultButton(buttonOK);

        buttonOK.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onOK();
            }
        });

        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onCancel();
            }
        });

        // call onCancel() when cross is clicked
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                onCancel();
            }
        });

        // call onCancel() on ESCAPE
        contentPane.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onCancel();
            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

        btnEnvironmentDirectory.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                FileChooserDescriptor fileChooserDescriptor = FileChooserDescriptorFactory.createSingleFolderDescriptor();
                Project project = browser.getProject();
                final FileChooserDialog chooser = FileChooserFactory.getInstance().createFileChooser(fileChooserDescriptor,
                        WindowManagerEx.getInstanceEx().suggestParentWindow(project));
                VirtualFile[] files = chooser.choose(project.getBaseDir(), project);
                VirtualFile virtualDirectory = files[0];
                String directory = virtualDirectory.getPath();
                tfEnvironmentDirectory.setText(directory);
            }
        });
    }

    private void onOK() {
        try {
            browser.addStore(tfEnvironmentDirectory.getText(), tfStoreName.getText());
        } catch (Exception e) {
            JXErrorPane.showDialog(browser.getRootComponent(), new ErrorInfo(
                    "Add Berkeley DB (Java Edition) store",
                    e.getMessage(),
                    null, "ERROR", e, ErrorLevel.SEVERE, null)
            );
        }
        dispose();
    }

    private void onCancel() {
        dispose();
    }
}
