package com.kalixia.intellij.plugins.je.browser;

import com.intellij.openapi.components.ApplicationComponent;
import com.intellij.openapi.components.PersistentStateComponent;
import com.intellij.openapi.components.State;
import com.intellij.openapi.components.Storage;
import org.jdom.Element;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

@State(name = "JESettings", storages = {@Storage(id = "je", file = "$APP_CONFIG$/je.xml")})
public class Settings implements ApplicationComponent, PersistentStateComponent<Element> {
    private List<StoreConfiguration> stores = new ArrayList<StoreConfiguration>();

    public Element getState() {
        @NonNls Element element = new Element("je");
        for (StoreConfiguration store : stores) {
            @NonNls Element child = new Element("store");
            child.setAttribute("directory", store.getEnvironmentDirectory());
            child.setAttribute("name", store.getStoreName());
            element.addContent(child);
        }
        return element;
    }

    @SuppressWarnings({"unchecked"})
    public void loadState(Element element) {
        for (@NonNls Element child : (Iterable<? extends Element>) element.getChildren("store")) {
            String directory = child.getAttributeValue("directory");
            String name = child.getAttributeValue("name");
            addStore(directory, name);
        }
    }

    public void addStore(String directory, String name) {
        stores.add(new StoreConfiguration(directory, name));
    }

    public void removeStore(String name) {
        for (StoreConfiguration store : stores) {
            if (store.getStoreName().equals(name)) {
                stores.remove(store);
                return;
            }
        }
    }

    public List<StoreConfiguration> getStores() {
        return stores;
    }

    @NonNls
    @NotNull
    public String getComponentName() {
        return "JE Settings";
    }

    public void initComponent() {
    }

    public void disposeComponent() {
    }

    public class StoreConfiguration implements Serializable {
        private final String environmentDirectory;
        private final String storeName;

        public StoreConfiguration(String environmentDirectory, String storeName) {
            this.environmentDirectory = environmentDirectory;
            this.storeName = storeName;
        }

        public String getEnvironmentDirectory() {
            return environmentDirectory;
        }

        public String getStoreName() {
            return storeName;
        }
    }
}
