package com.kalixia.intellij.plugins.je.browser.table;

import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.project.Project;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.persist.EntityCursor;
import com.sleepycat.persist.EntityIndex;
import com.sleepycat.persist.model.EntityModel;
import com.sleepycat.persist.raw.RawField;
import com.sleepycat.persist.raw.RawObject;
import com.sleepycat.persist.raw.RawStore;
import org.jdesktop.swingx.JXErrorPane;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

class JEEntityLargeTableModel extends AbstractTableModel implements TableModel {
    private EntityIndex<Object, RawObject> index;
    private List<Object> keys;
    private RawField[] fields;
    private final Logger logger = Logger.getInstance(getClass().getName());

    public JEEntityLargeTableModel(Project project, RawStore store, final String clazz) {
        EntityModel model = store.getModel();
        Collection<RawField> values = model.getRawType(clazz).getFields().values();
        this.fields = values.toArray(new RawField[values.size()]);

        try {
            this.index = store.getPrimaryIndex(clazz);
            ProgressManager.getInstance().runProcessWithProgressSynchronously(new Runnable() {
                public void run() {
                    ProgressIndicator indicator = ProgressManager.getInstance().getProgressIndicator();
                    indicator.setText("Loading primary keys for entity " + clazz + "...");
                    indicator.setFraction(0);
                    EntityCursor cursor = null;
                    try {
                        cursor = index.keys();
                        keys = new ArrayList<Object>();
                        long total = index.count();
                        double position = 1;
                        for (Object key : cursor) {
                            keys.add(key);
                            indicator.setFraction(position / total);
                        }
                    } catch (DatabaseException e) {
                        JXErrorPane.showDialog(e);
                    } finally {
                        if (cursor != null) {
                            try {
                                cursor.close();
                            } catch (DatabaseException e) {
                                logger.error("Can't close cursor", e);
                            }
                        }
                    }
                }
            }, "Berkeley DB (Java Edition)", false, project);
        } catch (DatabaseException e) {
            JXErrorPane.showDialog(e);
        }
    }

    public int getRowCount() {
        return keys.size();
    }

    public int getColumnCount() {
        return fields.length;
    }

    @Override
    public String getColumnName(int i) {
        return fields[i].getName();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        try {
            RawObject current = index.get(keys.get(rowIndex));
            Collection<Object> values = current.getValues().values();
            Object[] data = values.toArray(new Object[values.size()]);
            return data[columnIndex];
        } catch (DatabaseException e) {
            logger.error("Can't fetch cursor", e);
            return null;
        }
    }


}