package com.kalixia.intellij.plugins.je.browser.table;

import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.project.Project;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.persist.EntityCursor;
import com.sleepycat.persist.model.EntityModel;
import com.sleepycat.persist.raw.RawField;
import com.sleepycat.persist.raw.RawObject;
import com.sleepycat.persist.raw.RawStore;
import org.jdesktop.swingx.JXErrorPane;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class JEEntityTableModel extends AbstractTableModel implements TableModel {
    private RawField[] fields;
    private List<RawObject> entities;

    public JEEntityTableModel(Project project, final RawStore store, final String clazz) {
        EntityModel model = store.getModel();
        Collection<RawField> values = model.getRawType(clazz).getFields().values();
        this.fields = values.toArray(new RawField[values.size()]);
        this.entities = new ArrayList<RawObject>();
        ProgressManager.getInstance().runProcessWithProgressSynchronously(new Runnable() {
            public void run() {
                ProgressIndicator indicator = ProgressManager.getInstance().getProgressIndicator();
                indicator.setText("Loading entities for class " + clazz + "...");
                indicator.setFraction(0);
                EntityCursor<RawObject> cursor = null;
                try {
                    cursor = store.getPrimaryIndex(clazz).entities();
                    long total = store.getPrimaryIndex(clazz).count();
                    double position = 1;
                    for (RawObject entity : cursor) {
                        entities.add(entity);
                        indicator.setFraction(position / total);
                    }
                } catch (DatabaseException e) {
                    JXErrorPane.showDialog(e);
                } finally {
                    try {
                        if (cursor != null) cursor.close();
                    } catch (Exception e) {
                        JXErrorPane.showDialog(e);
                    }
                }
            }
        }, "Berkeley DB (Java Edition)", false, project);
    }

    public int getColumnCount() {
        return fields.length;
    }

    public int getRowCount() {
        return entities.size();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        RawObject current = entities.get(rowIndex);
        Collection<Object> values = current.getValues().values();
        Object[] data = values.toArray(new Object[values.size()]);
        return data[columnIndex];
    }

    @Override
    public String getColumnName(int i) {
        return fields[i].getName();
    }
}
