package com.kalixia.intellij.plugins.je.browser.tree;

import com.intellij.openapi.util.IconLoader;
import com.sleepycat.persist.model.PrimaryKeyMetadata;
import com.sleepycat.persist.model.SecondaryKeyMetadata;
import com.sleepycat.persist.raw.RawStore;
import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import java.awt.*;

public class JEBrowserTreeCellRenderer extends DefaultTreeCellRenderer {
    private JLabel lblStore = new JLabel(STORE_ICON);
    private JLabel lblEntity = new JLabel(ENTITY_ICON);
    private JLabel lblPrimarykey = new JLabel(PRIMARY_KEY_ICON);
    private JLabel lblSecondaryKey = new JLabel(SECONDARY_KEY_ICON);
    private static final Icon STORE_ICON = IconLoader.findIcon("/javaee/persistenceUnit.png");
    private static final Icon ENTITY_ICON = IconLoader.findIcon("/javaee/persistenceEntity.png");
    private static final Icon PRIMARY_KEY_ICON = IconLoader.findIcon("/nodes/dataPkColumn.png");
    private static final Icon SECONDARY_KEY_ICON = IconLoader.findIcon("/nodes/dataFkColumn.png");

    @Override
    public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
        if (node instanceof StoreTreeNode) {
            StoreTreeNode storeNode = (StoreTreeNode) node;
            lblStore.setText(storeNode.getStore().getStoreName());
            return lblStore;
        } else if (node instanceof EntityTreeNode) {
            EntityTreeNode entityNode = (EntityTreeNode) node;
            String clazz = entityNode.getClazz();            
            long entitiesCount = 0;
            try {
                RawStore store = entityNode.getStore();
                entitiesCount = store.getPrimaryIndex(clazz).count();
            } catch (Exception e) {}
            if (entitiesCount == 0)
                lblEntity.setText(clazz);
            else
                lblEntity.setText(clazz + " (" + entitiesCount + ')');
            return lblEntity;
        } else if (node instanceof PrimaryKeyMetadataTreeNode) {
            PrimaryKeyMetadataTreeNode primaryKeyNode = (PrimaryKeyMetadataTreeNode) node;
            lblPrimarykey.setText(primaryKeyNode.getPrimaryKey().getName());
            return lblPrimarykey;
        } else if (node instanceof SecondaryKeyMetadataTreeNode) {
            SecondaryKeyMetadataTreeNode secondaryKeyNode = (SecondaryKeyMetadataTreeNode) node;
            lblSecondaryKey.setText(secondaryKeyNode.getSecondaryKey().getKeyName());
            return lblSecondaryKey;
        } else {
            return super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
        }
    }
}
