package com.kalixia.intellij.plugins.je.environment;

import com.intellij.openapi.components.ApplicationComponent;
import com.intellij.openapi.fileEditor.FileEditor;
import com.intellij.openapi.fileEditor.FileEditorPolicy;
import com.intellij.openapi.fileEditor.FileEditorProvider;
import com.intellij.openapi.fileEditor.FileEditorState;
import com.intellij.openapi.fileEditor.FileEditorStateLevel;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.sleepycat.je.EnvironmentConfig;
import org.jdom.Element;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import java.util.Properties;
import java.io.IOException;

public class EnvironmentConfigFileEditorProvider implements ApplicationComponent, FileEditorProvider {
    private EnvironmentConfiguration editor;

    public boolean accept(@NotNull Project project, @NotNull VirtualFile virtualFile) {
        return "je.properties".equals(virtualFile.getName());
    }

    @NotNull
    public FileEditor createEditor(@NotNull Project project, @NotNull VirtualFile virtualFile) {
        Properties properties = new Properties();
        try {
            properties.load(virtualFile.getInputStream());
        } catch (IOException e) {
            System.err.println("Can't load je.properties file!");
        }
        editor = new EnvironmentConfiguration(new EnvironmentConfig(properties));
        return editor;
    }

    public void disposeEditor(@NotNull FileEditor fileEditor) {
        //todo:((EnvironmentConfigurationOld) fileEditor).dispose();
    }

    @NotNull
    public FileEditorState readState(@NotNull Element element, @NotNull Project project, @NotNull VirtualFile virtualFile) {
        return new FileEditorState() {
            public boolean canBeMergedWith(FileEditorState otherState, FileEditorStateLevel level) {
                return false;
            }
        };
    }

    public void writeState(@NotNull FileEditorState fileEditorState, @NotNull Project project, @NotNull Element element) {
    }

    @NotNull @NonNls
    public String getEditorTypeId() {
        return "JE";
    }

    @NotNull
    public FileEditorPolicy getPolicy() {
        return FileEditorPolicy.PLACE_BEFORE_DEFAULT_EDITOR;
    }

    @NonNls @NotNull
    public String getComponentName() {
        return "Berkeley DB (JE) Environment Configuration Editor Provider";
    }

    public void initComponent() {
    }

    public void disposeComponent() {
    }
}
