package com.kalixia.intellij.plugins.je.environment;

import com.kalixia.intellij.plugins.je.environment.JEDatabaseConfigurationForm;
import com.intellij.codeHighlighting.BackgroundEditorHighlighter;
import com.intellij.ide.structureView.StructureViewBuilder;
import com.intellij.openapi.fileEditor.FileEditor;
import com.intellij.openapi.fileEditor.FileEditorLocation;
import com.intellij.openapi.fileEditor.FileEditorState;
import com.intellij.openapi.fileEditor.FileEditorStateLevel;
import com.intellij.openapi.util.UserDataHolderBase;
import com.sleepycat.je.DbInternal;
import com.sleepycat.je.EnvironmentConfig;
import com.sleepycat.je.ExceptionListener;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import java.util.Properties;

public class EnvironmentConfiguration extends UserDataHolderBase implements FileEditor {
    private EnvironmentConfig config;
    private Properties properties;
    private JEDatabaseConfigurationForm form;

    public EnvironmentConfiguration(EnvironmentConfig config) {
        this.config = config;
        this.properties = DbInternal.getProps(config);
    }

    @NotNull
    public JComponent getComponent() {
        if (form == null) {
            form = new JEDatabaseConfigurationForm();
            form.setData(this);
        }
        return form.getRootComponent();
    }

    @Nullable
    public JComponent getPreferredFocusedComponent() {
        return form.getRootComponent();
    }

    @NonNls
    @NotNull
    public String getName() {
        return "Berkeley DB (JE)";
    }

    @NotNull
    public FileEditorState getState(@NotNull FileEditorStateLevel fileEditorStateLevel) {
        return new FileEditorState() {
            public boolean canBeMergedWith(FileEditorState fileEditorState, FileEditorStateLevel fileEditorStateLevel) {
                return false;
            }
        };
    }

    public void setState(@NotNull FileEditorState fileEditorState) {
        // todo: code this method
    }

    public boolean isModified() {
        return form != null && form.isModified(this);
    }

    public boolean isValid() {
        return true;
    }

    public void selectNotify() {
    }

    public void deselectNotify() {
    }

    public void addPropertyChangeListener(@NotNull PropertyChangeListener propertyChangeListener) {
    }

    public void removePropertyChangeListener(@NotNull PropertyChangeListener propertyChangeListener) {
    }

    @Nullable
    public BackgroundEditorHighlighter getBackgroundHighlighter() {
        return null;
    }

    public FileEditorLocation getCurrentLocation() {
        return null;
    }

    @Nullable
    public StructureViewBuilder getStructureViewBuilder() {
        return null;
    }

    public void dispose() {
        this.config = null;
    }


    // Delegate settings to the underlying {@link EnvironmentConfig}.


    public boolean getAllowCreate() {
        return config.getAllowCreate();
    }

    public ExceptionListener getExceptionListener() {
        return config.getExceptionListener();
    }

    public boolean getLocking() {
        return config.getLocking();
    }

    public long getLockTimeout() {
        return config.getLockTimeout();
    }

    public boolean getReadOnly() {
        return config.getReadOnly();
    }

    public boolean getTransactional() {
        return config.getTransactional();
    }

    public boolean getTxnSerializableIsolation() {
        return config.getTxnSerializableIsolation();
    }

    public long getTxnTimeout() {
        return config.getTxnTimeout();
    }

    public void setAllowCreate(boolean b) {
        config.setAllowCreate(b);
    }

    public void setConfigParam(String s, String s1) throws IllegalArgumentException {
        config.setConfigParam(s, s1);
    }

    public void setExceptionListener(ExceptionListener exceptionListener) {
        config.setExceptionListener(exceptionListener);
    }

    public void setLocking(boolean b) {
        config.setLocking(b);
    }

    public void setLockTimeout(long l) throws IllegalArgumentException {
        config.setLockTimeout(l);
    }

    public void setReadOnly(boolean b) {
        config.setReadOnly(b);
    }

    public void setTransactional(boolean b) {
        config.setTransactional(b);
    }

    public void setTxnSerializableIsolation(boolean b) {
        config.setTxnSerializableIsolation(b);
    }

    public void setTxnTimeout(long l) throws IllegalArgumentException {
        config.setTxnTimeout(l);
    }

    public int getCachePercent() {
        return config.getCachePercent();
    }

    public long getCacheSize() {
        return config.getCacheSize();
    }

    public String getConfigParam(String s) throws IllegalArgumentException {
        return config.getConfigParam(s);
    }

    public boolean getTxnNoSync() {
        return config.getTxnNoSync();
    }

    public boolean getTxnWriteNoSync() {
        return config.getTxnWriteNoSync();
    }

    public void setCachePercent(int i) throws IllegalArgumentException {
        config.setCachePercent(i);
    }

    public void setCacheSize(long l) throws IllegalArgumentException {
        config.setCacheSize(l);
    }

    public void setTxnNoSync(boolean b) {
        config.setTxnNoSync(b);
    }

    public void setTxnWriteNoSync(boolean b) {
        config.setTxnWriteNoSync(b);
    }

    public boolean getLogInConsole() {
        return Boolean.valueOf(properties.getProperty("java.util.logging.ConsoleHandler.on"));
    }

    public void setLogInConsole(boolean b) {
        config.setConfigParam("java.util.logging.ConsoleHandler.on", Boolean.toString(b));
    }

    public boolean getLogInFile() {
        return Boolean.valueOf(properties.getProperty("java.util.logging.FileHandler.on"));
    }

    public void setLogInFile(boolean b) {
        config.setConfigParam("java.util.logging.FileHandler.on", Boolean.toString(b));
    }

    public String getLogFileCount() {
        return properties.getProperty("java.util.logging.FileHandler.count");
    }

    public void setLogFileCount(String value) {
        config.setConfigParam("java.util.logging.FileHandler.count", value);
    }

    public String getLogFileLimit() {
        return properties.getProperty("java.util.logging.FileHandler.limit");
    }

    public void setLogFileLimit(String value) {
        config.setConfigParam("java.util.logging.FileHandler.limit", value);
    }
}
