package com.kalixia.intellij.plugins.je.environment;

import com.kalixia.intellij.plugins.je.environment.EnvironmentConfiguration;
import javax.swing.JTabbedPane;
import javax.swing.JPanel;
import javax.swing.JCheckBox;
import javax.swing.JSpinner;
import javax.swing.JComboBox;
import javax.swing.JTextField;

public class JEDatabaseConfigurationForm {
    private JTabbedPane tabs;
    private JPanel configurationPanel;
    private JTabbedPane environmentPanel;
    private JCheckBox allowCreateField;
    private JCheckBox readOnlyField;
    private JCheckBox allowLockingField;
    private JCheckBox transactionalField;
    private JSpinner lockingTimeoutField;
    private JSpinner transactionsTimeoutField;
    private JCheckBox transactionsSerializableIsolationField;
    private JCheckBox transactionsSynchronizationField;
    private JCheckBox transactionsWriteNoSyncField;
    private JSpinner cachePercentageField;
    private JSpinner cacheSizeField;
    private JCheckBox logOnConsoleField;
    private JCheckBox logInDatabaseField;
    private JCheckBox logInFileField;
    private JComboBox logLevelField;
    private JSpinner experimentalAdler32;
    private JPanel environmentTab;
    private JPanel storeTab;
    private JPanel experimentalTab;
    private JTextField logFileCountField;
    private JTextField logFileLimitField;

    public JPanel getRootComponent() {
        return configurationPanel;
    }

    public void setData(EnvironmentConfiguration data) {
        allowCreateField.setSelected(data.getAllowCreate());
        readOnlyField.setSelected(data.getReadOnly());
        allowLockingField.setSelected(data.getLocking());
        transactionalField.setSelected(data.getTransactional());
        logOnConsoleField.setSelected(data.getLogInConsole());
        logInFileField.setSelected(data.getLogInFile());
        logFileCountField.setText(data.getLogFileCount());
        logFileLimitField.setText(data.getLogFileLimit());
        transactionsSerializableIsolationField.setSelected(data.getTxnSerializableIsolation());
        transactionsSynchronizationField.setSelected(data.getTxnNoSync());
        transactionsWriteNoSyncField.setSelected(data.getTxnWriteNoSync());
    }

    public void getData(EnvironmentConfiguration data) {
        data.setAllowCreate(allowCreateField.isSelected());
        data.setReadOnly(readOnlyField.isSelected());
        data.setLocking(allowLockingField.isSelected());
        data.setTransactional(transactionalField.isSelected());
        data.setLogInConsole(logOnConsoleField.isSelected());
        data.setLogInFile(logInFileField.isSelected());
        data.setLogFileCount(logFileCountField.getText());
        data.setLogFileLimit(logFileLimitField.getText());
        data.setTxnSerializableIsolation(transactionsSerializableIsolationField.isSelected());
        data.setTxnNoSync(transactionsSynchronizationField.isSelected());
        data.setTxnWriteNoSync(transactionsWriteNoSyncField.isSelected());
    }

    public boolean isModified(EnvironmentConfiguration data) {
        if (allowCreateField.isSelected() != data.getAllowCreate()) return true;
        if (readOnlyField.isSelected() != data.getReadOnly()) return true;
        if (allowLockingField.isSelected() != data.getLocking()) return true;
        if (transactionalField.isSelected() != data.getTransactional()) return true;
        if (logOnConsoleField.isSelected() != data.getLogInConsole()) return true;
        if (logInFileField.isSelected() != data.getLogInFile()) return true;
        if (logFileCountField.getText() != null ? !logFileCountField.getText().equals(data.getLogFileCount()) : data.getLogFileCount() != null)
            return true;
        if (logFileLimitField.getText() != null ? !logFileLimitField.getText().equals(data.getLogFileLimit()) : data.getLogFileLimit() != null)
            return true;
        if (transactionsSerializableIsolationField.isSelected() != data.getTxnSerializableIsolation()) return true;
        if (transactionsSynchronizationField.isSelected() != data.getTxnNoSync()) return true;
        if (transactionsWriteNoSyncField.isSelected() != data.getTxnWriteNoSync()) return true;
        return false;
    }
}
